package edu.fudan.cube;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JPanel;

/**
 * Util widget for color choosing.
 * 
 * @author Lei Huang
 */
public class ColorChooser extends JPanel {
    JButton chooseButton;
    Color color;
    List<ColorChangedListener> listeners = new ArrayList<ColorChangedListener>();

    public ColorChooser(Color color) {
        this.color = color;
        chooseButton = new JButton();
        chooseButton.setText("RGB: " + color.getRed() + " " + color.getGreen() + " " + color.getBlue());
        chooseButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                showDialog();
            }
        });
        add(chooseButton);
    }

    private void showDialog() {
        color = JColorChooser.showDialog(this, "Choose a color", color);
        chooseButton.setText("RGB: " + color.getRed() + " " + color.getGreen() + " " + color.getBlue());
        invokeListeners();
    }

    private void invokeListeners() {
        for(ColorChangedListener listener : listeners) {
            listener.colorChanged(color);
        }
    }

    public Color getColor() {
        return color;
    }

    public void addColorChangedListener(ColorChangedListener listener) {
        listeners.add(listener);
    }
}
