package edu.fudan.cube;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.image.BufferedImage;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import edu.fudan.cube.core.Cube;
import edu.fudan.cube.core.Point;

/**
 * The main frame and entry point of the 3D project.
 * 
 * @author Lei Huang
 */
public class MainFrame extends JFrame {
	private Canvas canvas; /* canvas to draw the cube */
	private JPanel menuPanel; /* menu */
	private JSlider slider; /* slider for the rotation speed */
	private ColorChooser[] colorChooser; /* choose the color of the sides */
	private Cube cube; /* the cube */
	private Timer timer; /* timer to rotate the cube */
    private int mouseX = 200;
    private int mouseY = 200;
    
    public MainFrame() {
        cube = Cube.createCube();

		// generate canvas
        canvas = new CubeCanvas();
        canvas.setBackground(Color.WHITE);
        add(canvas, BorderLayout.CENTER);

		// generate menu
        menuPanel = new JPanel();
        menuPanel.setSize(150, 400);
        menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.Y_AXIS));
        
        JPanel panel = new JPanel();
        panel.add(menuPanel);
        add(panel, BorderLayout.EAST);

        JPanel titlePanel = new JPanel();
        titlePanel.add(new JLabel("Set colors for sides"));
        menuPanel.add(titlePanel);
        JPanel subtitlePanel = new JPanel();
        subtitlePanel.add(new JLabel("(black for none)"));
        menuPanel.add(subtitlePanel);

		// generate six ColorChooser for six sides.
        colorChooser = new ColorChooser[6];
        for(int i=0; i<6; i++) {
            colorChooser[i] = new ColorChooser(cube.getSideColor()[i]);
            colorChooser[i].addColorChangedListener(new ColorChangedInvoker(i));
            menuPanel.add(colorChooser[i]);
        }

		// generate slider to adjust the speed of rotation.
        JPanel titleSlidePanel = new JPanel();
        titleSlidePanel.add(new JLabel("Slide for faster rotate"));
        menuPanel.add(titleSlidePanel);

        slider = new JSlider(20, 100, 60);
        menuPanel.add(slider);
        slider.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent changeEvent) {
                Point.rotateSpeed = 120 - slider.getValue();
            }
        });

        setTitle("3D Cube");
        setSize(600, 430);
        setVisible(true);
        setDefaultCloseOperation(EXIT_ON_CLOSE);

        canvas.addMouseMotionListener(new MouseMotionAdapter() {
            @Override
            public void mouseMoved(MouseEvent mouseEvent) {
				// fetch the new mouse position
                mouseX = mouseEvent.getX() - canvas.getX();
                mouseY = mouseEvent.getY() - canvas.getY();
            }
        });
        
        canvas.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseExited(MouseEvent mouseEvent) {
				// do not rotate when mouse is out of the canvas
                mouseX = 200;
                mouseY = 200;
            }
        });
        
        timer = new Timer();
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
				// do the rotation every 40ms.
                cube = cube.rotate((mouseX - 200) / 100.0, (mouseY - 200) / 100.0);
                canvas.repaint();
            }
        }, 40, 40);
    }

    class CubeCanvas extends Canvas {
        @Override
		/**
		 * update the canvas use double buffer method.
		 */
        public void update(Graphics g) {
            Dimension dimension = this.getSize();
			// the buffer
            BufferedImage bufferImage = new BufferedImage(dimension.width, dimension.height, BufferedImage.TYPE_3BYTE_BGR);
            Graphics buffer = bufferImage.getGraphics();

			// clear buffer
            buffer.setColor(getBackground());
            buffer.fillRect(0, 0, dimension.width, dimension.height);

			// draw the cube
            cube.draw(bufferImage, null);

			// apply the buffer
            g.drawImage(bufferImage, 0, 0, this);
        }

        @Override
        public void paint(Graphics g) {
            update(g);
        }
    }

    public static void main(String args[]) {
        new MainFrame();
    }

    private class ColorChangedInvoker implements ColorChangedListener {
        private int index;

        public ColorChangedInvoker(int i) {
            index = i;
        }

        @Override
        public void colorChanged(Color color) {
            if(color.equals(Color.BLACK)) {
                cube.setSideColor(index, null);
            } else {
                cube.setSideColor(index, color);
            }
        }
    }
}
