package edu.fudan.cube.core;

import java.awt.Color;
import java.awt.image.BufferedImage;

import edu.fudan.cube.DrawUtil3D;

/**
 * Line is connected with two point.
 * 
 * @author Lei Huang
 */
public class Line implements GraphComposite {
    private Point[] points;

    public Line() {
        points = new Point[2];
        points[0] = new Point(0, 0, 0);
        points[1] = new Point(0, 0, 0);
    }
    
    public Line(Point[] points) {
        this.points = points;
    }
    
	/**
	 * Draw the line on the image.
	 */
    @Override
    public void draw(BufferedImage image, Color color) {
		// do projection
        Point a = DrawUtil3D.project(points[0]);
        Point b = DrawUtil3D.project(points[1]);

        if(Math.abs(a.getX() - b.getX()) == 0 && Math.abs(a.getY() - b.getY()) == 0) {
			// two points are same
            return;
        } else if(Math.abs(a.getX() - b.getX()) > Math.abs(a.getY() - b.getY())) {
			// let a be the left point
            if(a.getX() > b.getX()) {
                Point c = a;
                a = b;
                b = c;
            }
            for(int x = (int)Math.round(a.getX()); x <= Math.round(b.getX()); x++) {
				// get y and draw
                int y = (int)Math.abs((a.getX() - x) * (b.getY() - a.getY()) / (a.getX() - b.getX()) + a.getY());
                DrawUtil3D.drawPixel(image, x, y, color);
            }
        } else {
			// let a be the top point
            if(a.getY() > b.getY()) {
                Point c = a;
                a = b;
                b = c;
            }
            for(int y = (int)Math.round(a.getY()); y <= Math.round(b.getY()); y++) {
				// get x and draw
                int x = (int)Math.abs((a.getY() - y) * (b.getX() - a.getX()) / (a.getY() - b.getY()) + a.getX());
                DrawUtil3D.drawPixel(image, x, y, color);
            }
        }
    }
}
