package edu.fudan.cube.core;

/**
 * 3D Point
 * 
 * @author Lei Huang
 */
public class Point implements Comparable<Point> {
    private double x;
    private double y;
    private double z;

    public static double rotateSpeed = 60;

    public Point(double x, double y, double z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }

    public double getX() {
        return x;
    }

    public void setX(double x) {
        this.x = x;
    }

    public double getY() {
        return y;
    }

    public void setY(double y) {
        this.y = y;
    }

    public double getZ() {
        return z;
    }

    public void setZ(double z) {
        this.z = z;
    }

    public Point rotate(double dx, double dy) {
        if(dx == 0 && dy == 0) {
            return new Point(getX(), getY(), getZ());
        }
        double x = dy;
        double y = -dx;
        double z = 0;
        double theta = Math.hypot(x, y);
        x /= theta;
        y /= theta;
        theta /= rotateSpeed;
        double cos = Math.cos(theta);
        double sin = Math.sin(theta);

        return new Point(
                (cos+(1-cos)*x*x)*getX() + ((1-cos)*x*y-sin*z)*getY() + ((1-cos)*x*z+sin*y)*getZ(),
                ((1-cos)*x*y+sin*z)*getX() + (cos+(1-cos)*y*y)*getY() + ((1-cos)*y*z-sin*x)*getZ(),
                ((1-cos)*z*x-sin*y)*getX() + ((1-cos)*z*y+sin*x)*getY() + (cos+(1-cos)*z*z)*getZ()
        );
    }

    @Override
    public int compareTo(Point point) {
        double delta = getZ() - point.getZ();
        if(delta > 0) return -1;
        else if(delta < 0) return 1;
        else return 0;
    }
}
