package edu.fudan.poly;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;

import edu.fudan.poly.core.Polygon;

/**
 * Main frame and entry point of our project.
 * 
 * @author Lei Huang
 */
public class MainFrame extends JFrame {
	Canvas canvas; /* main canvas */
	List<Polygon> polygonList = new ArrayList<Polygon>(); /* polygons */
	EditState state = EditState.NORMAL; /* initial state */

	MouseActionListener mouseListener;
	Polygon curPolygon;
	int curPointIndex;

	public MainFrame() {
		// generate new canvas
		canvas = new MyCanvas();
		add(canvas, BorderLayout.CENTER);

		// apply mouse operation on the MouseActionListener
		mouseListener = new MouseActionListener();
		canvas.addMouseListener(mouseListener);
		canvas.addMouseMotionListener(mouseListener);

		setSize(430, 430);
		setTitle("Polygon");
		setVisible(true);
	}

	class MyCanvas extends Canvas {
		@Override
		public void update(Graphics g) {
			// get the buffer image and draw it to the canvas
			Dimension dimension = this.getSize();
			BufferedImage image = ImageWrapper.get(dimension);
			g.drawImage(image, 0, 0, this);
		}

		@Override
		public void paint(Graphics g) {
			update(g);
		}
	}

	class MouseActionListener implements MouseListener, MouseMotionListener {
		@Override
		public void mouseDragged(MouseEvent event) {
			Point mousePoint = event.getPoint();
			mousePoint.translate(-canvas.getX(), -canvas.getY());

			if (state == EditState.EDIT_POLY) {
				curPolygon.setPoint(curPointIndex, mousePoint);
			}

			canvas.repaint();
		}

		@Override
		public void mouseMoved(MouseEvent event) {
			// TODO Auto-generated method stub

		}

		@Override
		public void mouseClicked(MouseEvent event) {
			// TODO Auto-generated method stub

		}

		@Override
		public void mouseEntered(MouseEvent event) {
			// TODO Auto-generated method stub

		}

		@Override
		public void mouseExited(MouseEvent event) {
			// TODO Auto-generated method stub

		}

		@Override
		public void mousePressed(MouseEvent event) {
			// get the position of the mouse
			int mouseX = event.getX() - canvas.getX();
			int mouseY = event.getY() - canvas.getY();

			if (state == EditState.NORMAL) {
				if (event.getButton() == MouseEvent.BUTTON1) {
					// drag the original points
					for (Polygon polygon : polygonList) {
						int point = polygon.getPoint(mouseX, mouseY);
						if (point != -1) {
							curPolygon = polygon;
							curPointIndex = point;
							// the drag operation is deprecated since there is
							// some bugs on it.
							/* state = EditState.EDIT_POLY; */
							return;
						}
					}
					// create a new polygon
					Point point = new Point(mouseX, mouseY);
					curPointIndex = 0;
					curPolygon = new Polygon(point);
					polygonList.add(curPolygon);
					state = EditState.NEW_POLY;
				}
			} else if (state == EditState.NEW_POLY) {
				if (event.getButton() == MouseEvent.BUTTON3) {
					// right click to the normal
					state = EditState.NORMAL;
				} else {
					// left click to append new points.
					Point point = new Point(mouseX, mouseY);
					curPolygon.addPoint(point);
					curPointIndex++;
				}
			}

			canvas.repaint();
		}

		@Override
		public void mouseReleased(MouseEvent event) {
			if (state == EditState.EDIT_POLY) {
				state = EditState.NORMAL;
			}
		}
	}

	public static void main(String args[]) {
		new MainFrame();
	}
}
