package edu.fudan.poly.core;

import java.awt.Point;
import java.awt.image.BufferedImage;

public class PolygonUtil {

	/**
	 * Draw a triangle, the Point C is the new point.
	 */
	public static void drawNewTriangle(Point a, Point b, Point c, BufferedImage image) {
		// get the range of X
		int minX = (int) Math.round(Math.min(Math.min(a.getX(), b.getX()), c.getX()));
		int maxX = (int) Math.round(Math.max(Math.max(a.getX(), b.getX()), c.getX()));

		for (int x = minX; x <= maxX; x++) {
			// get the range of Y
			double minY = 1e100;
			double maxY = -1e100;
			minY = Math.min(minY, getMinY(a, b, x));
			maxY = Math.max(maxY, getMaxY(a, b, x));
			minY = Math.min(minY, getMinY(a, c, x));
			maxY = Math.max(maxY, getMaxY(a, c, x));
			minY = Math.min(minY, getMinY(b, c, x));
			maxY = Math.max(maxY, getMaxY(b, c, x));

			boolean first = true;
			for (int y = (int) Math.round(minY); first || y < (int) Math.round(maxY); y++) {
				// do the XOR operation. If the original color is black then set
				// to white, vise versa.
				int rgb = image.getRGB(x, y);
				if (rgb == -16777216) {
					rgb = -1;
				} else {
					rgb = 0;
				}
				image.setRGB(x, y, rgb);
				first = false;
			}
		}

		// fix the line of the origin edge.
		for (int x = minX; x <= maxX; x++) {
			double minY = getMinY(a, b, x);
			double maxY = getMaxY(a, b, x);

			if (minY > maxY || minY < 0 || maxY > 500) {
				continue;
			}

			for (int y = (int) Math.round(minY); y <= (int) Math.round(maxY); y++) {
				fix(x, y, image);
			}
		}
	}

	public static void drawModifiedTriangle(Point a, Point b, Point c, Point d, BufferedImage image) {
		drawNewTriangle(a, b, c, image);
		drawNewTriangle(a, b, d, image);
	}

	private static void fix(int x, int y, BufferedImage image) {
		// get near eight color of the polygon and set color to the one which is
		// more.
		int c[] = new int[8];
		c[0] = image.getRGB(x - 1, y - 1);
		c[1] = image.getRGB(x - 1, y);
		c[2] = image.getRGB(x - 1, y + 1);
		c[3] = image.getRGB(x, y - 1);
		c[4] = image.getRGB(x, y + 1);
		c[5] = image.getRGB(x + 1, y - 1);
		c[6] = image.getRGB(x + 1, y);
		c[7] = image.getRGB(x + 1, y + 1);
		
		// count the numbers
		int p = 0;
		for (int i = 0; i < 8; i++) {
			if(c[i] == -1) p++;
			else p--;
		}

		// do the fix
		if (p > 0) {
			image.setRGB(x, y, -1);
		} else if (p < 0) {
			image.setRGB(x, y, -16777216);
		}
	}

	/**
	 * Get the range of Y of edge from point a to point b at the x-axis is at x.
	 */
	private static double getMaxY(Point a, Point b, int x) {
		if (a.getX() > b.getX()) {
			// make sure a.x is less than b.x
			Point c = a;
			a = b;
			b = c;
		}
		if (a.getX() > x || b.getX() < x) {
			// out of bound
			return -1e100;
		}
		if (a.getX() == b.getX()) {
			// vertical edge
			return Math.max(a.getY(), b.getY());
		}
		double d = (b.getY() - a.getY()) / (b.getX() - a.getX());
		double y = (x - a.getX()) * d + a.getY();
		if (x > a.getX() && x < b.getX()) {
			y += 0.5 * Math.abs(d);
		}
		return y;
	}

	/**
	 * Get the range of Y of edge from point a to point b at the x-axis is at x.
	 */
	private static double getMinY(Point a, Point b, int x) {
		if (a.getX() > b.getX()) {
			// make sure a.x is less than b.x
			Point c = a;
			a = b;
			b = c;
		}
		if (a.getX() > x || b.getX() < x) {
			// out of bound
			return 1e100;
		}
		if (a.getX() == b.getX()) {
			// vertical edge
			return Math.min(a.getY(), b.getY());
		}
		double d = (b.getY() - a.getY()) / (b.getX() - a.getX());
		double y = (x - a.getX()) * d + a.getY();
		if (x > a.getX() && x < b.getX()) {
			y -= 0.5 * Math.abs(d);
		}
		return y;
	}
}
